﻿using ServiceCenter.Logs;
using SqlSugar;
using WCS.Core;

namespace WCS.WorkEngineering.Extensions
{
    /// <summary>
    ///  检查中心
    /// </summary>
    public class CheckCenter<T>
    {
        /// <summary>
        /// 使用此检查中心的世界
        /// </summary>
        private World World { get; set; }

        private List<T> WhereList = new List<T>();

        private List<ChcekErrorInfo<T>> WhereFuncs = new List<ChcekErrorInfo<T>>();

        /// <summary>
        ///  初始化
        /// </summary>
        /// <param name="world"></param>
        /// <param name="whereList"></param>
        public CheckCenter(World world, List<T> whereList)
        {
            World = world;
            WhereList = whereList;
        }

        /// <summary>
        ///  错误检查
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="exFunc"></param>
        /// <param name="errorMessage"></param>
        /// <param name="level"></param>
        public CheckCenter<T> ChcekError(Func<T, bool> exFunc, string errorMessage, LogLevelEnum level)
        {
            WhereFuncs.Add(new ChcekErrorInfo<T>(exFunc, errorMessage, level));
            return this;
        }

        /// <summary>
        ///  错误检查
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="exFunc"></param>
        /// <param name="errorMessage"></param>
        public CheckCenter<T> ChcekError(Func<T, bool> exFunc, string errorMessage) => ChcekError(exFunc, errorMessage, LogLevelEnum.Mid);

        /// <summary>
        ///  执行所有检查命令
        /// </summary>
        /// <returns></returns>
        public bool ExecuteCommand()
        {
            bool result = true;
            foreach (var item in WhereFuncs)
            {
                Func<T, bool> exp = item.Func;
                var list = WhereList.Where(exp).ToList();
                if (list.Any())
                {
                    list.ForEach(it => World.Log($"{item.ErrorMessage}", item.Level));
                    result = false;
                }
            }

            return result;
        }
    }

    /// <summary>
    ///  错误检查信息
    /// </summary>
    /// <typeparam name="T">被检查数据</typeparam>
    public class ChcekErrorInfo<T>
    {
        /// <summary>
        ///  检查条件
        /// </summary>
        public Func<T, bool> Func { get; set; }

        /// <summary>
        ///  错误时记录的信息
        /// </summary>
        public string ErrorMessage { get; set; }

        /// <summary>
        ///  错误级别
        /// </summary>
        public LogLevelEnum Level { get; set; }

        /// <summary>
        ///
        /// </summary>
        /// <param name="func">检查条件</param>
        /// <param name="errorMessage">错误时记录的信息</param>
        /// <param name="level">错误级别</param>
        public ChcekErrorInfo(Func<T, bool> func, string errorMessage, LogLevelEnum level)
        {
            Func = func;
            ErrorMessage = errorMessage;
            Level = level;
        }
    }
}